#!/usr/bin/env python
# -*- coding: ascii -*-
#
#  gbottler.py - a SSTP Bottle Client
#  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: gbottler.py,v 1.35 2004/04/25 20:11:29 atzm Exp $
#

import os
import sys
import time
import string
import re
import mimetools
import StringIO
import gettext
import getopt
from types import *

import gtk

from bottlelib import BottleClient, BottleClientError
from logs import LogManager, LogWindow
from phrases import PhraseManager, PhraseWindow
import textmanager
import prefs
import queryluid

try:
	from ninix.script import *
except:
	from script import *

from bottleparser import *

gettext.install('gbottler')

LIBDIR = "~/.bottlecase"

# vote types
VOTE  = "Vote"
AGREE = "Agree"

APP = 'GBottler'
VER = '0.0.4'

class Bottler(BottleClient):
	def __init__(self, app):
		BottleClient.__init__(self, app.pbar)
		self.app = app

	def handle_sstp_message(self, message, unicast, forced):
		if not message:
			return
		# dispatch SSTP message to local SSTP server
		ifghost = re.search("IfGhost: ?(.+?)\r\n", message).group(1)

		# append channel information to Sender
		m = re.split("(Sender: ?.+?)\r\n", message)
		m.insert(2, " / " + ifghost + "\r\n")
		message = string.join(m, "")

		if self.app.prefs.get('forward'):
			if not self.app.prefs.get('aa'):
				if self.app.sakuraname == ifghost or \
					   self.app.target_ghost == ifghost or \
					   self.app.target_ghost == Application.DEFAULT_TARGET:
					BottleClient.handle_sstp_message(self, message, unicast, forced,
													 host=self.app.host, port=self.app.port)
			elif self.app.prefs.get('foig'):
				if self.app.sakuraname in self.app.local_ghost or \
				   self.app.target_ghost in self.app.local_ghost:
					BottleClient.handle_sstp_message(self, message, unicast, forced,
													 host=self.app.host, port=self.app.port)
			else:
				BottleClient.handle_sstp_message(self, message, unicast, forced,
												 host=self.app.host, port=self.app.port)

		# parse SSTP message
		file = StringIO.StringIO(message)
		request = file.readline()
		headers = mimetools.Message(file)
		charset = headers.get("charset", "Shift_JIS")
		if unicast:
			channel = "unicast"
		else:
			mid = self.headers["mid"]
			if forced:
				channel = "broadcast"
			else:
				channel = self.headers["channel"]
		script = headers["script"]

		# log message
		self.app.log_message(mid, channel, ifghost, script)

		if self.app.prefs.get('auto update'):
			self.app.log_window.update(None, None)

		#print "Debug msg: ", ifghost, ", ", script, "\n", message

	def handle_dialog_message(self, message):
		self.app.monitor_clear()
		self.app.monitor_insert(message)
		self.app.window.bell()

	def handle_broadcast_information(self, type, forced):
		BottleClient.handle_broadcast_information(self, type, forced)
		if type in [VOTE, AGREE]:
			self.app.log_votes(
				self.headers["mid"], type, int(self.headers["num"]))

	def handle_all_users(self):
		self.app.show_users()

	def handle_channel_users(self, channel):
		pass

	def handle_close_channel(self, channel):
		self.app.close_channel(channel)

class Application:
	def edit_cut(self, data, widget):
		self.main_text.emit("cut-clipboard")

	def edit_copy(self, data, widget):
		self.main_text.emit("copy-clipboard")

	def edit_paste(self, data, widget):
		self.main_text.emit("paste-clipboard")

	def edit_delete(self, data, widget):
		self.main_text.emit("delete-from-cursor", gtk.DELETE_CHARS, 1)

	def get_icon_button(self, stock, size=gtk.ICON_SIZE_SMALL_TOOLBAR, relief=gtk.RELIEF_NONE, label=None):
		i = gtk.Image()
		i.set_from_stock(stock, size)
		i.show()

		b = gtk.Button()
		b.set_relief(relief)
		b.show()

		if label:
			h = gtk.HBox(gtk.FALSE, 0)
			h.show()
			h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)

			l = gtk.Label(label)
			l.show()
			h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
			b.add(h)
		else:
			b.add(i)

		return b

	DEFAULT_TARGET = unicode(_("default"), "utf-8")
	def __init__(self, host, port):

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.quit, "WM destroy")
		self.window.set_title(unicode(_("GBottler"), "utf-8"))

		self.set_self_messages()
		menu_items = self.get_menu_items()

		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
		self.window.add(self.main_vbox)
		self.main_vbox.show()

		accel_group = gtk.AccelGroup()

		self.item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
		self.item_factory.create_items(menu_items)

		self.window.add_accel_group(accel_group)

		self.menu_list = {}
		for m in menu_items:
			key = re.sub("_", "", m[0])
			self.menu_list[key] = self.item_factory.get_item(key)
		for m in ["/%s/%s" % (self.msg["file"][1], self.msg["part"]),
				  "/%s" % self.msg["channel"][1],
				  "/%s" % self.msg["ghost"][1]]:
			self.menu_list[m].set_sensitive(gtk.FALSE)			
			
		self.menubar = self.item_factory.get_widget("<main>")

		self.main_vbox.pack_start(self.menubar, gtk.FALSE, gtk.TRUE, 0)
		self.menubar.show()
		# End of creating Menubar

		self.tooltips = gtk.Tooltips()
		checkButton = self.get_icon_button(gtk.STOCK_SPELL_CHECK,
										   gtk.ICON_SIZE_BUTTON)
		#								   label=unicode(_("Check"), "utf-8"))
		checkButton.connect("clicked", self.check, None)
		self.tooltips.set_tip(checkButton, unicode(_("Check"), "utf-8"))

		testButton = self.get_icon_button(gtk.STOCK_REFRESH,
										  gtk.ICON_SIZE_BUTTON)
		#								  label=unicode(_("Test"), "utf-8"))
		testButton.connect("clicked", self.test, None)
		self.tooltips.set_tip(testButton, unicode(_("Test"), "utf-8"))

		self.send_button = self.get_icon_button(gtk.STOCK_OK,
												gtk.ICON_SIZE_BUTTON)
		#										label=unicode(_("Send"), "utf-8"))
		self.send_button.connect("clicked", self.send, None)
		self.tooltips.set_tip(self.send_button, unicode(_("Send"), "utf-8"))

		channelLabel = gtk.Label("  " + unicode(_("Channel") + ": ", "utf-8"))
		self.channel = gtk.Label()
		self.c_e_box = gtk.EventBox()
		self.c_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.c_e_box.add(self.channel)
		self.c_e_box.show()

		self.channel_ghost = gtk.Label()
		self.g_e_box       = gtk.EventBox()
		self.g_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.g_e_box.add(self.channel_ghost)
		self.g_e_box.show()

		usersLabel = gtk.Label("  " + unicode(_("Users") + ": ", "utf-8"))
		self.users = gtk.Label()

		group2 = gtk.HBox(gtk.FALSE, 0)
		group2.set_border_width(1)
		self.main_vbox.pack_start(group2, gtk.FALSE, gtk.TRUE, 0)
		group2.show()

		group2.pack_start(checkButton, gtk.FALSE, gtk.TRUE, 0)
		group2.pack_start(testButton, gtk.FALSE, gtk.TRUE, 0)
		group2.pack_start(self.send_button, gtk.FALSE, gtk.TRUE, 0)

		group2.pack_start(channelLabel, gtk.FALSE, gtk.TRUE, 0)
		group2.pack_start(self.c_e_box, gtk.FALSE, gtk.TRUE, 0)
		group2.pack_start(self.g_e_box, gtk.FALSE, gtk.TRUE, 0)
		group2.pack_start(usersLabel, gtk.FALSE, gtk.TRUE, 0)
		group2.pack_start(self.users, gtk.FALSE, gtk.TRUE, 0)

		checkButton.show()
		testButton.show()
		self.send_button.set_sensitive(gtk.FALSE)
		self.send_button.show()

		channelLabel.show()
		self.channel.show()
		self.channel_ghost.show()
		usersLabel.show()
		self.users.show()

		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.main_text = gtk.TextView()
		self.main_text.set_cursor_visible(gtk.TRUE)
		self.main_text.set_size_request(500,100)
		self.editor = self.main_text.get_buffer()
		self.editor.set_text(r"\t\u\s[10]\h\s[0]\e")
		self.main_text.set_editable(gtk.TRUE)
		self.main_text.set_wrap_mode(gtk.WRAP_CHAR)
		sw.add(self.main_text)
		sw.show()
		self.main_text.show()
		self.main_text.emit("move-cursor", gtk.MOVEMENT_LOGICAL_POSITIONS, -2, gtk.FALSE)

		self.frame = gtk.Frame()
		self.frame.set_shadow_type(gtk.SHADOW_IN)
		self.frame.add(sw)
		self.frame.show()

		self.pbar = gtk.ProgressBar()
		self.pbar.set_size_request(500, 20)
		self.pbar.show()

		self.paned = gtk.VPaned()
		self.paned.pack1(self.frame)
		self.paned.pack2(self.pbar)
		self.main_vbox.pack_start(self.paned, gtk.TRUE, gtk.TRUE, 0)
		self.paned.show()

		pmsg = self.get_popup_menu_messages()
		popup_menu_items = self.get_popup_menu_items(pmsg)

		self.popup_item_factory = gtk.ItemFactory(gtk.Menu, "<main>", accel_group)
		self.popup_item_factory.create_items(popup_menu_items)
		self.popup_menu = self.popup_item_factory.get_widget("<main>")
		self.main_text.connect("button-press-event", self.popup)

		self.window.show()
		self.main_text.grab_focus()

		self.client = Bottler(self)
		self.client.debug = 2

		self.parser = BottleParser('strict')
		# this parser was using check script

		if host:
			self.host = host
		else:
			self.host = "localhost"
		if port:
			self.port = port
		else:
			self.port = 9801
		self.local_ghost_last_notified = 0
		self.target_ghost = ""

		self.to_sjis = lambda x: x.encode("sjis")
		self.sjis_to = lambda x: unicode(x, "sjis")

		self.luid_info = None
		try:
			luid = open(os.path.join(open_bottlecase(), "luid")).read()
		except IOError:
			luid = None
		if luid and len(luid) == 82:
			self.client.luid = luid
			self.luid_info = queryluid.LUIDInfo(luid)

		self.execute_command = 'GetNames'
		self.sakuraname = ""
		#self.keroname = ""

		self.logger = LogManager(VOTE, AGREE)
		self.log_window = LogWindow(self, self.logger, VOTE, AGREE)
		if self.logger.is_disabled():
			d = gtk.Dialog(unicode(_("Warning"), "utf-8"), parent=self.window,
						   flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_MODAL)
			d.connect("destroy", lambda w: w.destroy)
			d.set_border_width(10)
			l = gtk.Label(unicode(_("Warning"), "utf-8") + ": " + \
						  unicode(_("another bottler.py compatible SSTP bottle client is running!"), "utf-8") + "\n\n" + \
						  unicode(_("logging disabled."), "utf-8"))
			l.set_justify(gtk.JUSTIFY_RIGHT)
			l.show()
			d.vbox.add(l)
			d.add_button(unicode(_("OK"), "utf-8"), True)
			d.show()
			d.run()
			d.destroy()

		self.phrase_manager = PhraseManager()
		self.phrase_window = PhraseWindow(self, self.phrase_manager)

		self.menu_list["/%s/%s" % (self.msg["window"][1],
								   self.msg["logs"])].connect("activate", self.log_window.open, None)

		self.menu_list["/%s/%s" % (self.msg["window"][1],
								   self.msg["phrases"])].connect("activate", self.phrase_window.open, None)

		self.about = None

		self.local_ghost = []
		self.local_ghost_last_notified = 0
		self.send_ghost_names()

		self.prefs = prefs.PreferenceManager()

		# initialize
		self.c_e_id = 1 # channel label eventbox handler id
		self.g_e_id = 1
		self.disable_channel()

		self.timeout_id = None

		self.textmanager = textmanager.TextManager(self, self.main_text)
		self.textmanager.set_style_script()
		self.editor.connect('end-user-action', self.textmanager.coloring)
		self.textmanager.coloring()
		self.color_parser = BottleParser('loose')
		# this parser was using coloring

	def get_popup_menu_messages(self):
		pmsg = {
     		"sessions":	    unicode(_("Sessions"), "utf-8"),
     		"timecritical": unicode(_("TimeCritical"), "utf-8"),
     		"quick":		unicode(_("Quick"), "utf-8"),
     		"syncronized":  unicode(_("Syncronized"), "utf-8"),
     
     		"sakura":       unicode(_("Sakura"), "utf-8"),
     		"countenance":  unicode(_("Countenance"), "utf-8"),
     		"general":	    unicode(_("General"), "utf-8"),
     		"shy":		    unicode(_("Shy"), "utf-8"),
     		"surprised":	unicode(_("Surprised"), "utf-8"),
     		"angst":		unicode(_("Angst"), "utf-8"),
     		"ease off":	    unicode(_("Ease off"), "utf-8"),
     		"smile":		unicode(_("Smile"), "utf-8"),
     		"closed eyes":  unicode(_("Closed eyes"), "utf-8"),
     		"angry":		unicode(_("Angry"), "utf-8"),
     		"scorned":	    unicode(_("Scorned"), "utf-8"),
     		"signboard":	unicode(_("Signboard"), "utf-8"),
     		"singing":	    unicode(_("Singing"), "utf-8"),
     
     		"unyu":		    unicode(_("Unyu"), "utf-8"),
     		"opened eyes":  unicode(_("Opened Eyes"), "utf-8"),
     
     		"wait":		    unicode(_("Wait"), "utf-8"),
     		"sec":		    unicode(_(" sec"), "utf-8"),
     
     		"meta string":  unicode(_("Meta String"), "utf-8"),
     		"user name":	unicode(_("User name"), "utf-8"),
     		"self name":	unicode(_("Self name"), "utf-8"),
     		"true":		    unicode(_("True"), "utf-8"),
     		"false":		unicode(_("False"), "utf-8"),
     		"kero name":	unicode(_("Kero name"), "utf-8"),
     
     		"new line":	    unicode(_("New line"), "utf-8"),
     		"clear":		unicode(_("Clear"), "utf-8"),
     
     		"terminate":	unicode(_("Terminate"), "utf-8"),
     		}
		return pmsg
	
	def get_popup_menu_items(self, pmsg):     
		popup_menu_items = (
     		( "/sep0", None, None, 0, "<Separator>" ),
     		( "/%s" % pmsg["sessions"], None, None, 0, "<Branch>" ),
     		( "/%s/%s" % (pmsg["sessions"], pmsg["timecritical"]), None, self.insert_t, 0, None ),
     		( "/%s/%s" % (pmsg["sessions"], pmsg["quick"]), None, self.insert_q, 0, None ),
     		( "/%s/%s" % (pmsg["sessions"], pmsg["syncronized"]), None, self.insert_s, 0, None ),
     		( "/sep1", None, None, 0, "<Separator>" ),
     
     		( "/%s" % pmsg["sakura"], None, self.insert_h, 0, None ),
     		( "/%s" % pmsg["countenance"], None, None, 0, "<Branch>" ),
     		( "/%s/[0]  %s" % (pmsg["countenance"], pmsg["general"]), "<control><alt>0", self.insert_s0,  0, None, ),
     		( "/%s/[1]  %s" % (pmsg["countenance"], pmsg["shy"]), "<control><alt>1", self.insert_s1, 0, None, ),
     		( "/%s/[2]  %s" % (pmsg["countenance"], pmsg["surprised"]), "<control><alt>2", self.insert_s2, 0, None, ),
     		( "/%s/[3]  %s" % (pmsg["countenance"], pmsg["angst"]), "<control><alt>3", self.insert_s3, 0, None, ),
     		( "/%s/[4]  %s" % (pmsg["countenance"], pmsg["ease off"]), "<control><alt>4", self.insert_s4, 0, None, ),
     		( "/%s/[5]  %s" % (pmsg["countenance"], pmsg["smile"]), "<control><alt>5", self.insert_s5, 0, None, ),
     		( "/%s/[6]  %s" % (pmsg["countenance"], pmsg["closed eyes"]), "<control><alt>6", self.insert_s6, 0, None, ),
     		( "/%s/[7]  %s" % (pmsg["countenance"], pmsg["angry"]), "<control><alt>7", self.insert_s7, 0, None, ),
     		( "/%s/[8]  %s" % (pmsg["countenance"], pmsg["scorned"]), "<control><alt>8", self.insert_s8, 0, None, ),
     		( "/%s/[20] %s" % (pmsg["countenance"], pmsg["signboard"]), None, self.insert_s20, 0, None, ),
     		( "/%s/[25] %s" % (pmsg["countenance"], pmsg["singing"]), None, self.insert_s25, 0, None, ),
     		( "/sep2", None, None, 0, "<Separator>" ),
     
     		( "/%s" % pmsg["unyu"], None, self.insert_u, 0, None ),
     		( "/%s" % pmsg["countenance"], None, None, 0, "<Branch>" ),
     		( "/%s/[10] %s" % (pmsg["countenance"], pmsg["general"]), "<control><alt><shift>g", self.insert_s10, 0, None ),
     		( "/%s/[11] %s" % (pmsg["countenance"], pmsg["opened eyes"]), None, self.insert_s11, 0, None ),
     		( "/%s/[19] %s" % (pmsg["countenance"], pmsg["singing"]), None, self.insert_s19, 0, None ),
     		( "/sep3", None, None, 0, "<Separator>" ),
     
     		( "/%s" % pmsg["wait"], None, None, 0, "<Branch>" ),
     		( "/%s/0.1%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w2, 0, None ),
     		( "/%s/0.2%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w4, 0, None ),
     		( "/%s/0.3%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w6, 0, None ),
     		( "/%s/0.4%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w8, 0, None ),
     		( "/%s/0.5%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w10, 0, None ),
     		( "/%s/0.6%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w12, 0, None ),
     		( "/%s/0.9%s" % (pmsg["wait"], pmsg["sec"]), "<control><alt>w", self.insert_w18, 0, None ),
     		( "/%s/1.0%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w20, 0, None ),
     		( "/%s/1.2%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w24, 0, None ),
     		( "/%s/1.5%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w30, 0, None ),
     		( "/%s/2.0%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w40, 0, None ),
     		( "/%s/2.5%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w50, 0, None ),
     		( "/%s/3.0%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w60, 0, None ),
     
     		( "/%s" % pmsg["meta string"], None, None, 0, "<Branch>" ),
     		( "/%s/%s" % (pmsg["meta string"], pmsg["user name"]), None, self.insert_username, 0, None ),
     		( "/%s/%s (%s)" % (pmsg["meta string"], pmsg["self name"], pmsg["true"]), None, self.insert_selfname, 0, None ),
     		( "/%s/%s (%s)" % (pmsg["meta string"], pmsg["self name"], pmsg["false"]), None, self.insert_selfname2, 0, None),
     		( "/%s/%s" % (pmsg["meta string"], pmsg["kero name"]), None, self.insert_keroname, 0, None ),
     		( "/sep4", None, None, 0, "<Separator>" ),
     
     		( "/%s" % pmsg["new line"], "<control><alt>n", self.insert_n, 0, None ),
     		( "/%s" % pmsg["clear"], None, self.insert_c, 0, None ),
     		( "/URL", None, self.insert_url, 0, None ),
     
			( "/%s" % pmsg["terminate"], "<control><alt>e", self.insert_e, 0, None ),
     		)
		return popup_menu_items
	
	def set_self_messages(self):
		self.msg = {
			"file":   [unicode(_("File(_F)"), "utf-8"),
					   unicode(_("File(F)"), "utf-8")],
			"join":   unicode(_("Join"), "utf-8"),
			"part":   unicode(_("Part"), "utf-8"),
			"quit":   unicode(_("Quit"), "utf-8"),
			"edit":   [unicode(_("Edit(_E)"), "utf-8"),
					   unicode(_("Edit(E)"), "utf-8")],
			"cut":	  unicode(_("Cut"), "utf-8"),
			"copy":   unicode(_("Copy"), "utf-8"),
			"paste":  unicode(_("Paste"), "utf-8"),
			"delete": unicode(_("Delete"), "utf-8"),
			"clear":  unicode(_("Clear"), "utf-8"),
			"ca":	  unicode(_("Clear All"), "utf-8"),
			"pref":   [unicode(_("Preferences(_P)"), "utf-8"),
					   unicode(_("Preferences(P)"), "utf-8")],
			"channel":[unicode(_("Channel(_C)"), "utf-8"),
					   unicode(_("Channel(C)"), "utf-8")],
			"ghost":  [unicode(_("Ghost(_G)"), "utf-8"),
					   unicode(_("Ghost(G)"), "utf-8")],
			"window": [unicode(_("Window(_W)"), "utf-8"),
					   unicode(_("Window(W)"), "utf-8")],
			"logs":    unicode(_("Logs"), "utf-8"),
			"phrases": unicode(_("Phrases"), "utf-8"),
			"help":   [unicode(_("Help(_H)"), "utf-8"),
					   unicode(_("Help(H)"), "utf-8")],
			"about":  unicode(_("About"), "utf-8"),
		}
	
	def get_menu_items(self):
		menu_items = (
			( "/%s" % self.msg["file"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.msg["file"][1], self.msg["join"]), "<control>J", self.join, 0, None ),
			( "/%s/%s" % (self.msg["file"][1], self.msg["part"]), "<control>P", self.part, 0, None ),
			( "/%s/sep1" % self.msg["file"][1], None, None, 0, "<Separator>" ),
			( "/%s/%s" % (self.msg["file"][1], self.msg["quit"]), "<control>Q", self.quit, 0, None ),

			( "/%s" % self.msg["edit"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.msg["edit"][1], self.msg["cut"]), "<control>X", self.edit_cut, 0, None ),
			( "/%s/%s" % (self.msg["edit"][1], self.msg["copy"]), "<control>C", self.edit_copy, 0, None ),
			( "/%s/%s" % (self.msg["edit"][1], self.msg["paste"]), "<control>V", self.edit_paste, 0, None ),
			( "/%s/%s" % (self.msg["edit"][1], self.msg["delete"]), "<control>D", self.edit_delete,0, None ),
			( "/%s/sep4" % self.msg["edit"][1], None, None, 0, "<Separator>" ),

			( "/%s/%s" % (self.msg["edit"][1], self.msg["clear"]), None, self.edit_clear, 0, None ),
			( "/%s/%s" % (self.msg["edit"][1], self.msg["ca"]), None, self.edit_clear_all, 0, None ),
			( "/%s/sep5" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
			( "/%s/%s" % (self.msg["edit"][1], self.msg["pref"][0]), None, self.open_preference, 0, None ),

			( "/%s" % self.msg["channel"][0], None, None, 0, "<Branch>" ),
			( "/%s" % self.msg["ghost"][0], None, None, 0, "<Branch>" ),

			( "/%s" % self.msg["window"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.msg["window"][1], self.msg["logs"]), "<control>L", None, 0, None ),
			( "/%s/%s" % (self.msg["window"][1], self.msg["phrases"]), None, None, 0, None ),

			( "/%s" % self.msg["help"][0], None, None, 0, "<LastBranch>" ),
			( "/%s/%s" % (self.msg["help"][1], self.msg["about"]), None, self.about, 0, None ),
			)
	
		return menu_items

	def open_preference(self, widget=None, data=None):
		prefs.PreferenceDialog(self.window, self.prefs, self.luid_info)

	def main(self):
		gtk.main()
		return 0

	def popup(self, widget, event):
		if event.button == 3:
			self.popup_menu.popup(None, None, None, event.button, 0)

	def join(self, data, widget):
		if not self.client.initialized:
			try:
				self.client.init()
				self.luid_info = queryluid.LUIDInfo(self.client.luid)
			except BottleClientError, e:
				return
			filename = os.path.join(open_bottlecase(), "luid")
			try:
				open(filename, "w").write(self.client.luid)
			except IOError:
				self.monitor_insert("\n" + unicode(_("Warning"), "utf-8") + \
									": cannot write " + filename)
				self.window.bell()

		channels = ChannelDialog(self, self.window, self.client.channels).go()
		self.pbar.set_fraction(0.0)
		self.monitor_clear()

		if channels is None:
			return
		if self.client.join(channels):
			return

		self.pbar.set_fraction(0.0)
		self.monitor_clear()
		self.enable_channel()
		self.timeout_id = gtk.timeout_add(100, self.handle_event)

	def button_popup(self, widget, event, sender):
		if event.button in [1, 3]:
			sender.popup(None, None, None, event.button, 0)

	def enable_channel(self):
		channel_item = self.menu_list["/%s" % self.msg["channel"][1]]
		channel_item.deselect()
		channel_item.set_sensitive(gtk.FALSE)
		channel_item.remove_submenu()

		channels = []
		for n in self.client.in_channels:
			name = self.client.channels[n]["name"]
			channels.append(name)
			if not self.channel.get_text():
				self.channel.set_text(name)

		channel_menu = []
		for n in range(len(channels)):
			if n == 0:
				channel_menu.append(("/"+channels[n], None, None, 0, "<RadioItem>"))
			else:
				channel_menu.append(("/"+channels[n], None, None, 0, "/"+channels[n-1]))
		channel_menu = tuple(channel_menu)

		self.channel_factory = gtk.ItemFactory(gtk.Menu, "<main>")
		self.channel_factory.create_items(channel_menu)

		for n in range(len(channels)):
			self.channel_factory.get_item(channel_menu[n][0]).connect("toggled",
																	  self.update_channel,
																	  channels[n])

		channel_item.set_submenu(self.channel_factory.get_widget("<main>"))
		channel_item.set_sensitive(gtk.TRUE)

		if not self.c_e_box.handler_is_connected(self.c_e_id):
			self.c_e_id = self.c_e_box.connect("button-press-event", self.button_popup,
											   self.channel_factory.get_widget("<main>"))

		self.rebuild_ghostmenu()
		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["part"])].set_sensitive(gtk.TRUE)
		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["join"])].set_sensitive(gtk.FALSE)
		self.log_window.vb.set_sensitive(gtk.TRUE)
		self.log_window.ab.set_sensitive(gtk.TRUE)
		self.log_window.vi.set_sensitive(gtk.TRUE)
		self.log_window.ai.set_sensitive(gtk.TRUE)
		self.show_users()

	def update_channel(self, widget, data):
		self.channel.set_text(data)
		self.check_channel_permission()

	def rebuild_ghostmenu(self):
		ghost_item = self.menu_list["/%s" % self.msg["ghost"][1]]
		ghost_item.deselect()
		ghost_item.set_sensitive(gtk.FALSE)

		if self.g_e_box.handler_is_connected(self.g_e_id):
			self.g_e_box.disconnect(self.g_e_id)

		current = self.target_ghost
		ghosts = [self.DEFAULT_TARGET] + self.local_ghost
		#ghosts.append(self.sakuraname)
		name = ""
		for n in range(len(self.client.channels)):
			name = self.client.channels[n]["ghost"]
			if name and name not in ghosts:
				ghosts.append(name)

		ghost_menu = []
		for n in range(len(ghosts)):
			if n == 0:
				ghost_menu.append(("/"+ghosts[n], None, None, 0, "<RadioItem>"))
			else:
				ghost_menu.append(("/"+ghosts[n], None, None, 0, "/"+ghosts[n-1]))
		ghost_menu = tuple(ghost_menu)

		self.g_ifact = gtk.ItemFactory(gtk.Menu, "<main>")
		self.g_ifact.create_items(ghost_menu)

		all_ghost_items = {}
		for n in range(len(ghosts)):
			all_ghost_items[ghosts[n]] = self.g_ifact.get_item(ghost_menu[n][0])
			all_ghost_items[ghosts[n]].connect("toggled", self.update_ghost, ghosts[n])

		# reselect ghost it's selected by user
		if current in ghosts:
			all_ghost_items[current].set_active(gtk.TRUE)
			all_ghost_items[current].toggled()
		else:
			if current != self.DEFAULT_TARGET and current != name:
				all_ghost_items[self.sakuraname].set_active(gtk.TRUE)
				all_ghost_items[self.sakuraname].toggled()
			else:
				all_ghost_items[self.DEFAULT_TARGET].set_active(gtk.TRUE)
				all_ghost_items[self.DEFAULT_TARGET].toggled()

		ghost_item.set_submenu(self.g_ifact.get_widget("<main>"))
		ghost_item.set_sensitive(gtk.TRUE)

		if not self.g_e_box.handler_is_connected(self.g_e_id):
			self.g_e_id = self.g_e_box.connect("button-press-event", self.button_popup, self.g_ifact.get_widget("<main>"))

		self.check_channel_permission()

	def update_ghost(self, widget, data):
		self.target_ghost = data
		self.sakuraname = data
		self.channel_ghost.set_text("(" + self.target_ghost + ")")

	def check_channel_permission(self):
		channel = self.get_channel_id()
		if self.client.channels[channel]["nopost"]:
			self.send_button.set_sensitive(gtk.FALSE)
		else:
			self.send_button.set_sensitive(gtk.TRUE)

	def handle_event(self):
		if not self.client.connected:
			return
		handled = 0
		try:
			handled = self.client.handle_event()
		except BottleClientError, e:
			self.client.close()
			self.disable_channel()
			return

		now = time.time()
		if now - self.local_ghost_last_notified > 30: # about 30 seconds
			self.local_ghost_last_notified = now
			self.send_ghost_names()
			self.rebuild_ghostmenu()
		self.timeout_id = gtk.timeout_add(1000, self.handle_event)

	SSTP_EXECUTE = (
		"EXECUTE SSTP/1.0\r\n"
		"Sender: GBottler\r\n"
		"Command: %s\r\n"
		"Option: notranslate\r\n"
		"Charset: Shift_JIS\r\n"
		"\r\n")
	def send_ghost_names(self):
		try:
			data = self.client.send_sstp_message(self.SSTP_EXECUTE % self.execute_command,
												 self.host, self.port)
		except BottleClientError, e:
			return

		if data:
			if data[:12] == "SSTP/1.0 501":
				if self.execute_command == 'GetName':
					return # if GetName command was not implemented...

				#self.prefs.prefs['foig'].set_sensitive(gtk.FALSE)
				#if self.prefs.prefs['foig'].get_active():
				#	self.prefs.prefs['aa'].set_active(gtk.TRUE)
				#	self.prefs.save_database()
				# for old ninix-aya or other SSTP server,
				# pref 'forward only installed ghost' is
				# just for SSTP server implemented 'GetNames'

				self.execute_command = 'GetName'
				self.send_ghost_names()
				return
			elif data[:12] != "SSTP/1.0 200":
				return
		else:
			return

		pos = string.find(data, "\r\n\r\n")
		if pos < 0:
			return

		charset = string.split(data)[-1]
		if not charset:
			charset = 'sjis'

		res = ''
		try:
			res = unicode(data[pos:], charset)
		except TypeError:
			res = unicode(data[pos:], 'sjis')

		names = []
		for g in string.split(res, '\r\n'):
			if g and g[:7] != 'Charset':
				ghost = string.split(g, ',')
				names.append(ghost[0])

		if not names or self.local_ghost == names:
			return
		try:
			self.client.send_ghost_names(names)
		except BottleClientError, e:
			return
		if self.execute_command == 'GetNames':
			self.local_ghost = names
		elif self.execute_command == 'GetName':
			self.local_ghost = [string.split(names[0], ',')[0]]

		self.sakuraname = self.local_ghost[0]

	def monitor_clear(self):
		self.pbar.set_text("")

	def monitor_insert(self, text):
		setted = ""
		if self.pbar.get_text():
			setted = self.pbar.get_text()
		self.pbar.set_text(setted + text)

	def disable_channel(self):
		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["part"])].set_sensitive(gtk.FALSE)
		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["join"])].set_sensitive(gtk.TRUE)
		self.log_window.vb.set_sensitive(gtk.FALSE)
		self.log_window.ab.set_sensitive(gtk.FALSE)
		self.log_window.vi.set_sensitive(gtk.FALSE)
		self.log_window.ai.set_sensitive(gtk.FALSE)

		if self.c_e_box.handler_is_connected(self.c_e_id):
			self.c_e_box.disconnect(self.c_e_id)
			self.c_e_id = 1
		if self.g_e_box.handler_is_connected(self.g_e_id):
			self.g_e_box.disconnect(self.g_e_id)
			self.g_e_id = 1

		self.send_button.set_sensitive(gtk.FALSE)
		self.channel.set_text("")
		self.channel_ghost.set_text("")
		self.target_ghost = self.DEFAULT_TARGET
		self.users.set_text("")

		for m in ["/%s" % self.msg["ghost"][1],
				  "/%s" % self.msg["channel"][1]]:
			menu = self.menu_list[m]
			menu.deselect()
			menu.set_sensitive(gtk.FALSE)
			menu.remove_submenu()

	def get_channel_id(self):
		name = self.channel.get_text()
		for id in self.client.in_channels:
			if self.client.channels[id]["name"] == name:
				return id
		raise RuntimeError, "unknown channel name (%s)" % name

	def show_users(self):
		channel = self.get_channel_id()
		count = self.client.channels[channel]["count"]
		self.users.set_text("%d / %d " % (count, self.client.users))

	def part(self, data, widget):
		if not self.client.connected:
			return
		if self.timeout_id:
			gtk.timeout_remove(self.timeout_id)
			self.timeout_id = None
		self.client.close()
		self.logger.close()
		self.monitor_clear()
		self.monitor_insert(unicode(_("Connection closed."), "utf8"))
		self.disable_channel()

	def quit(self, data, widget):
		self.part(data, widget)
		gtk.mainquit()

	def edit_clear(self, data, widget):
		#self.monitor_clear()
		self.edit_clear_all(data, widget)
		self.textmanager.insert_with_color_from(r'\t\u\s[10]\h\s[0]\e')
		#self.editor.set_text(r"\t\u\s[10]\h\s[0]\e")
		#self.coloring()

	def edit_clear_all(self, data, widget):
		self.monitor_clear()
		self.editor.set_text("")

	def check(self, widget, data):
		self.main_text.grab_focus()
		self.monitor_clear()
		self.monitor_insert(unicode(_("Now checking script..."), "utf8"))
		[si, ei] = self.editor.get_bounds()
		lines = string.split(self.editor.get_text(si, ei), "\n")

		error = 0
		buffer = []
		i = 1.0 / len(lines)
		for n in range(len(lines)):
			if not lines[n]:
				break
			lines[n] = self.check_line(lines[n], n)
			if lines[n] is None:
				error = 1
				break
			buffer.extend(lines[n])
			self.pbar.set_fraction(self.pbar.get_fraction() + i)
		if error or self.check_tag_abuse(buffer):
			return 1
		else:
			self.pbar.set_fraction(0.0)
			self.monitor_insert(unicode(_("No error."), "utf8"))
			return 0

	def check_line(self, script, lineno):
		try:
			return self.parser.parse(script)
		except ParserError, e:
			self.pbar.set_fraction(0.0)
			self.monitor_insert(unicode(_("Error"), "utf8"))

			d = gtk.Dialog(title="Script Parser Error", parent=self.window, flags=1)
			d.set_has_separator(gtk.FALSE)
			d.connect("destroy", lambda w,data:d.destroy(), "WM destroy")
			d.vbox.set_border_width(10)

			h = gtk.HBox()
			i = gtk.Image()
			i.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_LARGE_TOOLBAR)
			i.show()
			h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)
			l = gtk.Label("Script Parser Error")
			l.show()
			h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
			h.show()
			d.vbox.add(h)
			s = gtk.HSeparator()
			s.show()
			d.vbox.add(s)

			s = gtk.ScrolledWindow()
			s.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
			t = gtk.TextView()
			t.set_cursor_visible(gtk.TRUE)
			t.set_size_request(300, 100)
			b = t.get_buffer()
			b.set_text('')
			tag = b.create_tag()
			tag.set_property('font', 'Monospace')

			b.insert_with_tags(b.get_end_iter(), unicode(_("Line"), "utf8") + \
							   " %d: %s\n%s\n" % (lineno, e.message, script) +
							   " " * e.column + "^" * (e.length or 1), tag)

			t.set_editable(gtk.FALSE)
			t.set_wrap_mode(gtk.WRAP_CHAR)
			s.add(t)

			f = gtk.Frame()
			f.set_shadow_type(gtk.SHADOW_IN)
			f.add(s)
			f.show()

			d.vbox.add(f)

			ok = gtk.Button(_("Close"), gtk.STOCK_CLOSE)
			ok.connect("clicked", lambda w,data:d.destroy(), None)
			ok.show()
			d.action_area.pack_start(ok, gtk.TRUE, gtk.TRUE, 0)

			t.show()
			s.show()
			d.show()

			return None

	RESERVED_SURFACES = (0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 19, 20, 25)

	def check_tag_abuse(self, script):
		scope = error = None
		surface_usage = [0, 10]

		msg_or  = unicode(_("or"), "utf8")
		msg_tag = unicode(_("tag"), "utf8")
		for node in script:
			if node[0] == SCRIPT_TAG:
				name, args = node[1], node[2:]
				if name == r"\h":
					scope = 0
				elif name == r"\u":
					scope = 1
				elif name == r"\s":
					try:
						id = int(args[0])
					except ValueError:
						error = unicode(_('bad use of surfaces'), 'utf-8')
						break
					if id in self.RESERVED_SURFACES:
						target = id / 10 % 2
					else:
						target = None
					if scope is None:
						if target is None:
							tag = r"\h %s \u %s" % (msg_or, msg_tag)
						elif target == 0:
							tag = r"\h %s" % msg_tag
						else:
							tag = r"\u %s" % msg_tag
						error = r"\s[%d] " % id + \
								unicode(_("is used without a preceding"), "utf8") + \
								" %s" % tag
						break
					if id in self.RESERVED_SURFACES:
						surface_usage[scope] = id
			elif node[0] == SCRIPT_TEXT:
				if scope is None:
					error = r"\h %s \u " % msg_or + \
							unicode(_("is needed before characters"), "utf8")
					break
		else:
			if surface_usage[0] / 10 % 2 == 1 or surface_usage[0] == 20 or \
			   surface_usage[1] / 10 % 2 == 0:
				error = unicode(_("bad use of surfaces"), "utf8")
		if error is not None:
			self.pbar.set_fraction(0.0)
			self.monitor_clear()
			self.monitor_insert(unicode(_("Error"), "utf8") + ": %s " % error)
			return 1
		return 0

	def test(self, widget, data):
		self.main_text.grab_focus()
		if self.check(widget, data):
			return
		self.send_local_message('[' + unicode(_("Test"), "utf-8") + ']', self.target_ghost, self.get_script())

	def send(self, widget, data):
		self.main_text.grab_focus()
		if self.check(widget, data):
			return

		d = gtk.Dialog(unicode(_("Really?"), "utf-8"), parent=self.window,
					   flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_MODAL)
		l = gtk.Label(unicode(_("Send to "), "utf8") + \
					  self.channel.get_text() + " / " + \
					  self.target_ghost + unicode(_(", OK?"), "utf8"))

		d.vbox.pack_start(l, gtk.TRUE, gtk.TRUE, 0)
		l.show()
		d.add_button(unicode(_("OK"), "utf8"), True)
		d.add_button(unicode(_("Cancel"), "utf8"), False)
		d.show()
		response = d.run()
		d.hide()
		d.destroy()

		if not response:
			return

		channel = self.get_channel_id()
		script = self.get_script()
		ghost = self.target_ghost
		if ghost == self.DEFAULT_TARGET:
			ghost = None
		self.monitor_clear()
		self.monitor_insert(unicode(_("Broadcasting..."), "utf8"))
		try:
			if self.client.send_broadcast(channel, script, ghost):
				error = self.client.headers["ExtraMessage"]
			else:
				error = None
		except BottleClientError, e:
			error = str(e)

		if error is not None:
			self.monitor_insert(unicode(_("failed!!")) + "\n" + \
								unicode(_("Error"), "utf8") + ": ")
			self.monitor_insert(unicode(error))
			self.window.bell()

		if self.prefs.get('ceas'):
			self.edit_clear(data, widget)

		self.pbar.set_fraction(0.0)

	def get_script(self):
		[si, ei] = self.editor.get_bounds()
		script = string.replace(self.editor.get_text(si, ei), "\n", "")

		if string.rstrip(script)[-2:] != r"\e":
			script = script + r"\e"
		return unicode(script, "utf-8")

	def about(self, data, widget):
		if self.about:
			return

		def aboutdestroy(widget, data):
			self.about.destroy()
			self.about = None

		self.about = gtk.Dialog(unicode(_("About"), "utf-8"), parent=self.window,
								flags=gtk.DIALOG_NO_SEPARATOR)
		self.about.connect("destroy", aboutdestroy, "WM destroy")
		self.about.set_border_width(10)

		h = gtk.HBox()
		i = gtk.Image()
		i.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_LARGE_TOOLBAR)
		i.show()
		h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)
		l = gtk.Label(unicode(_("About"), 'utf-8'))
		l.show()
		h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
		h.show()
		self.about.vbox.add(h)
		s = gtk.HSeparator()
		s.show()
		self.about.vbox.add(s)

		frame = gtk.Frame(label=" %s %s " % (APP, VER))
		l = gtk.Label("\n"+
					  " Copyright(C) 2001-'04, Tamito KAJIYAMA \n"+
					  " Copyright(C) 2003-'04, Atzm WATANABE \n")
		l.set_justify(gtk.JUSTIFY_RIGHT)
		l.show()
		frame.add(l)
		frame.show()
		self.about.vbox.add(frame)

		frame = gtk.Frame(label=unicode(_(" License "), "utf-8"))
		l = gtk.Label("\n"+
					  " GNU General Public License version 2 or later \n")
		l.set_justify(gtk.JUSTIFY_RIGHT)
		l.show()
		frame.add(l)
		frame.show()
		self.about.vbox.add(frame)

		b = gtk.Button(unicode(_("OK"), "utf8"), gtk.STOCK_OK)
		b.connect("clicked", aboutdestroy, None)
		b.show()
		self.about.action_area.pack_start(b, gtk.TRUE, gtk.TRUE, 0)

		self.about.show()

	def log_message(self, mid, channel, ifghost, script):
		self.logger.log_message(mid, channel, ifghost, script)

	def log_votes(self, mid, type, num):
		self.logger.log_votes(mid, type, num)

	SSTP_SEND = (
		"SEND SSTP/1.4\r\n"
		"Sender: SSTP Bottle / %s / %s\r\n"
		"IfGhost: %s\r\n"
		"Script: %s\r\n"
		"Option: notranslate\r\n"
		"Charset: Shift_JIS\r\n"
		"\r\n")
	def send_local_message(self, channel, ifghost, script):
		self.client.send_sstp_message(self.SSTP_SEND % (channel, ifghost, ifghost, script),
									  self.host, self.port)

	def vote_message(self, mid, type):
		if self.client.vote_message(mid, type):
			print "voting error"
			print self.client.headers["ExtraMessage"]
		self.monitor_clear()
		self.pbar.set_fraction(0.0)

	def insert(self, pre, post=""):
		self.main_text.emit("insert-at-cursor", pre)
		if post:
			self.main_text.emit("insert-at-cursor", post)
			self.main_text.emit("move-cursor", gtk.MOVEMENT_LOGICAL_POSITIONS,
								-1 * len(post), gtk.FALSE)

	def insert_h(self, data, widget): self.insert("\n\\h")
	def insert_u(self, data, widget): self.insert("\n\\u")
	def insert_s0(self, data, widget): self.insert(r"\s[0]")
	def insert_s1(self, data, widget): self.insert(r"\s[1]")
	def insert_s2(self, data, widget): self.insert(r"\s[2]")
	def insert_s3(self, data, widget): self.insert(r"\s[3]")
	def insert_s4(self, data, widget): self.insert(r"\s[4]")
	def insert_s5(self, data, widget): self.insert(r"\s[5]")
	def insert_s6(self, data, widget): self.insert(r"\s[6]")
	def insert_s7(self, data, widget): self.insert(r"\s[7]")
	def insert_s8(self, data, widget): self.insert(r"\s[8]")
	def insert_s20(self, data, widget): self.insert(r"\s[20]")
	def insert_s25(self, data, widget): self.insert(r"\s[25]")
	def insert_s10(self, data, widget): self.insert(r"\s[10]")
	def insert_s11(self, data, widget): self.insert(r"\s[11]")
	def insert_s19(self, data, widget): self.insert(r"\s[19]")
	def insert_username(self, data, widget): self.insert("%username")
	def insert_selfname(self, data, widget): self.insert("%selfname")
	def insert_selfname2(self, data, widget): self.insert("%selfname2")
	def insert_keroname(self, data, widget): self.insert("%keroname")
	def insert_url(self, data, widget): self.insert(r"\URL[http://", "]")
	def insert_c(self, data, widget): self.insert(r"\c")
	def insert_n(self, data, widget): self.insert("\\n\n")
	def insert_w2(self, data, widget): self.insert(r"\w2")
	def insert_w4(self, data, widget): self.insert(r"\w4")
	def insert_w6(self, data, widget): self.insert(r"\w6")
	def insert_w8(self, data, widget): self.insert(r"\w8")
	def insert_w10(self, data, widget): self.insert(r"\w8\w2")
	def insert_w12(self, data, widget): self.insert(r"\w8\w4")
	def insert_w18(self, data, widget): self.insert(r"\w9\w9")
	def insert_w20(self, data, widget): self.insert(r"\w8\w8\w4")
	def insert_w24(self, data, widget): self.insert(r"\w8\w8\w8")
	def insert_w30(self, data, widget): self.insert(r"\w9\w9\w9\w3")
	def insert_w40(self, data, widget): self.insert(r"\w9\w9\w9\w9\w4")
	def insert_w50(self, data, widget): self.insert(r"\w9\w9\w9\w9\w9\w5")
	def insert_w60(self, data, widget): self.insert(r"\w9\w9\w9\w9\w9\w9\w6")
	def insert_t(self, data, widget): self.insert(r"\t")
	def insert_q(self, data, widget): self.insert(r"\_q")
	def insert_s(self, data, widget): self.insert(r"\_s")
	def insert_e(self, data, widget): self.insert(r"\e")

class ChannelDialog:
	def __init__(self, app, master, channels):
		self.selection = []
		self.channels  = channels

		self.window = gtk.Dialog(parent=master, flags=1)
		self.window.set_has_separator(gtk.FALSE)
		self.window.connect("destroy", self.cancel, "WM destroy")
		self.window.set_title(unicode(_("Select Channels"), "utf-8"))

		self.window.vbox.set_border_width(10)

		cbs = []
		for n in range(len(self.channels)):
			cb = gtk.CheckButton(self.channels[n]["name"])
			cbs.append(cb)
			cb.connect("toggled", self.selected, [self.channels[n], n])
			self.window.vbox.pack_start(cb, gtk.TRUE, gtk.TRUE, 2)
			cb.show()

		self.window.vbox.show()

		self.buffer   = gtk.TextBuffer()
		self.textview = gtk.TextView(self.buffer)
		self.textview.set_editable(gtk.FALSE)
		self.textview.set_cursor_visible(gtk.FALSE)
		self.textview.set_justification(gtk.JUSTIFY_LEFT)
		self.textview.show()

		self.frame = gtk.Frame()
		self.frame.set_shadow_type(gtk.SHADOW_IN)
		self.frame.add(self.textview)
		self.frame.show()

		self.window.vbox.pack_start(self.frame, gtk.TRUE, gtk.TRUE, 2)

		bbox = gtk.HBox(gtk.FALSE, 1)
		bbox.set_border_width(2)
		ok = gtk.Button(unicode(_("OK"), "utf8"), gtk.STOCK_OK)
		ca = gtk.Button(unicode(_("Cancel"), "utf8"), gtk.STOCK_CANCEL)
		ok.connect("clicked", self.ok, None)
		ca.connect("clicked", self.cancel, None)
		bbox.pack_start(ok, gtk.TRUE, gtk.TRUE, 2)
		ok.show()
		bbox.pack_start(ca, gtk.TRUE, gtk.TRUE, 2)
		ca.show()
		self.window.vbox.pack_start(bbox, gtk.TRUE, gtk.TRUE, 2)
		bbox.show()

		self.window.show()

		cbs.reverse()
		for b in cbs:
			b.set_active(gtk.TRUE)

		ok.grab_focus()

	def selected(self, caller, data):
		n = data[1]
		data = data[0]
		buf = "info: " +  data["info"] + "\n" + "ghost: " + data["ghost"] + \
			  "\n" + "count: " + str(data["count"])
		self.buffer.set_text(buf)

		if n in self.selection:
			self.selection.remove(n)
		else:
			self.selection.append(n)
			self.selection.sort()

	def go(self):
		gtk.main()
		return self.selection

	def ok(self, widget, data):
		gtk.mainquit()
		self.window.hide()

	def cancel(self, widget, data):
		self.selection = None
		gtk.mainquit()
		self.window.hide()

def open_bottlecase(path=None):
	libdir = os.path.expanduser(path or LIBDIR)
	if not os.path.exists(libdir):
		os.mkdir(libdir)
	return libdir

def usage():
	port = unicode(_("alternative port for forwarding SSTP messages"), 'utf-8').encode('euc-jp')
	help = unicode(_("show this message"), 'utf-8').encode('euc-jp')
	host = unicode(_("alternative host for forwarding SSTP messages"), 'utf-8').encode('euc-jp')

	usage = """\
	-p, --port NUM  %s
	--host HOST     %s
	-h, --help      %s\n""" % (port, host, help)

	sys.stderr.write(unicode(_("Usage"), 'utf-8').encode('euc-jp') + ": " + \
					 os.path.basename(sys.argv[0]) + "\n" + \
					 unicode(_("Options"), 'utf-8').encode('euc-jp') + ":\n" + usage)
	sys.exit(1)

def main():
	# parse command line arguments
	try:
		options, rest = getopt.getopt(sys.argv[1:], "p:h",
									  ["port=", "help", "host="])
	except getopt.error, e:
		sys.stderr.write(unicode(_("Error"), 'utf-8').encode('euc-jp') + ": %s\n" % str(e))
		usage()
	if rest:
		usage()

	port = 9801
	host = ""   # default - localhost:9801

	# check environment variables
	val = os.environ.get("BOTTLER_PORT")
	if val:
		try:
			num = int(val)
		except ValueError:
			sys.stderr.write(unicode(_("Invalid BOTTLER_PORT number (ignored)") + \
									 "\n", 'utf-8').encode('euc-jp'))
		else:
			port = num

	# parse command line options
	for opt, val in options:
		if opt in ["-p", "--port"]:
			port = int(val)
		elif opt == "--host":
			host = val
		else:
			usage()
	# run
	app = Application(host, port)
	app.main()

if __name__ == "__main__":
	main()

