# -*- coding: ascii -*-
#
#  phrases.py - Phrase manipulator for GBottler
#  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: phrases.py,v 1.7 2004/04/25 07:09:42 atzm Exp $
#

import os, sys, re, string
import gtk

class PhraseManager:
	DBFILE = "phrases"
	DBWRITEFREQ = 10 # write phrases to file per 10 operations
	def __init__(self):
		from gbottler import open_bottlecase

		self.dbpath = os.path.join(open_bottlecase(), self.DBFILE)
		self.dblife = self.DBWRITEFREQ
		self.load_database()

	def load_database(self):
		self.phrases = []
		try:
			file = open(self.dbpath)
		except IOError:
			return
		while 1:
			line = unicode(file.readline(), "utf-8")
			if not line:
				break
			elif line[0] == "#":
				continue
			elif line[-2:] == "\r\n":
				line = line[:-2]
			elif line[-1] in ["\r", "\n"]:
				line = line[:-1]
			self.phrases.append(line)
		file.close()

	def save_database(self):
		try:
			file = open(self.dbpath, "w")
		except IOError:
			sys.stderr.write("Error: cannot write %s\n" % self.dbpath)
			return
		file.write("# This is an auto-generated file.\n")
		file.write("# Don't edit this file while running GBottler!\n")
		for line in self.phrases:
			file.write(line.encode("utf-8") + "\n")
		file.close()

	def close(self):
		self.save_database()

	def sync(self):
		self.dblife = self.dblife - 1
		if self.dblife == 0:
			self.dblife = self.DBWRITEFREQ
			self.save_database()

	def overwrite(self, phrases):
		self.phrases = phrases

	def list(self):
		return self.phrases[:]

	def get(self, index):
		return self.phrases[index]

	def append(self, phrase):
		self.phrases.append(phrase)
		self.sync()

	def insert(self, index, phrase):
		self.phrases.insert(index, phrase)
		self.sync()

	def modify(self, index, phrase):
		self.phrases[index] = phrase
		self.sync()

	def delete(self, index):
		del self.phrases[index]
		self.sync()

	def move_up(self, index):
		if index - 1 < 0:
			return
		self.phrases[index], self.phrases[index - 1] = \
							 self.phrases[index - 1], self.phrases[index]
		self.sync()

	def move_down(self, index):
		if index + 1 >= len(self.phrases):
			return
		self.phrases[index], self.phrases[index + 1] = \
							 self.phrases[index + 1], self.phrases[index]
		self.sync()

class PhraseWindow:
	def __init__(self, app, manager):
		self.app = app
		self.manager = manager
		# create GUI
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_size_request(350, 250)

		self.name_text = gtk.Entry()
		self.name_text.show()
		nf = gtk.Frame(unicode(_('Name'), 'utf-8'))
		nf.set_shadow_type(gtk.SHADOW_NONE)
		nf.show()
		nf.add(self.name_text)

		self.script_text = gtk.Entry()
		self.script_text.show()
		sf = gtk.Frame(unicode(_('Script'), 'utf-8'))
		sf.set_shadow_type(gtk.SHADOW_NONE)
		sf.show()
		sf.add(self.script_text)

		self.msg = {
			"close":  unicode(_("Close"), "utf-8"),
			"copy to main editor":   unicode(_("Copy to main editor"), "utf-8"),
			"cut":	unicode(_("Cut"), "utf-8"),
			"new":  unicode(_("New"), "utf-8"),
			"duplicate":  unicode(_("Duplicate"), "utf-8"),
			"up":  unicode(_("Up"), "utf-8"),
			"down":  unicode(_("Down"), "utf-8"),
			}

		menu_items = (
			( "/%s" % self.app.msg["file"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.app.msg["file"][1], self.msg["close"]), "<control>q", self.close, 0, None ),

			( "/%s" % self.app.msg["edit"][0], None, None, 0, "<Branch>" ),
			( "/%s/sep" % self.app.msg["edit"][1], None, None, 0, '<Separator>' ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy to main editor"]), None, self.edit_copy, 0, None ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["new"]), None, self.insert_new, 0, None ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["duplicate"]), None, self.edit_duplicate, 0, None ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.app.msg["delete"]), None, self.delete, 0, None ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["up"]), None, self.edit_up, 0, None ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["down"]), None, self.edit_down, 0, None ),

			( "/%s" % self.app.msg["help"][0], None, None, 0, "<LastBranch>" ),
			( "/%s/%s" % (self.app.msg["help"][1], self.app.msg["about"]), None, self.app.about, 0, None ),
			)

		self.window.connect("delete_event", self.close)
		self.window.set_title(unicode(_("GBottler"), "utf-8") + ":" + \
							  unicode(_("Phrases"), "utf-8"))

		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
		self.window.add(self.main_vbox)
		self.main_vbox.show()

		accel_group = gtk.AccelGroup()

		self.item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
		self.item_factory.create_items(menu_items)

		self.window.add_accel_group(accel_group)

		self.menu_list = {}
		for m in menu_items:
			key = re.sub("_", "", m[0])
			self.menu_list[key] = self.item_factory.get_item(key)

		self.popup_menu = self.item_factory.get_widget("/%s" % self.app.msg["edit"][1])

		self.menubar = self.item_factory.get_widget("<main>")
		self.main_vbox.pack_start(self.menubar, gtk.FALSE, gtk.TRUE, 0)
		self.menubar.show()
		# End of creating Menubar

		menubar2 = gtk.MenuBar()
		menubar2.show()

		# Toolbar
		self.tooltips = gtk.Tooltips()
		bf = gtk.Frame()
		bf.set_shadow_type(gtk.SHADOW_OUT)
		bf.show()
		bbox = gtk.HBox(gtk.FALSE, 0)
		bbox.show()
		bf.add(bbox)
		nb = self.app.get_icon_button(gtk.STOCK_NEW)
		self.tooltips.set_tip(nb, self.msg['new'])
		nb.connect('clicked', self.insert_new)
		bbox.pack_start(nb, gtk.FALSE, gtk.TRUE, 0)
		cb = self.app.get_icon_button(gtk.STOCK_COPY)
		self.tooltips.set_tip(cb, self.msg['duplicate'])
		cb.connect('clicked', self.edit_duplicate)
		bbox.pack_start(cb, gtk.FALSE, gtk.TRUE, 0)
		db = self.app.get_icon_button(gtk.STOCK_DELETE)
		self.tooltips.set_tip(db, self.app.msg['delete'])
		db.connect('clicked', self.delete)
		bbox.pack_start(db, gtk.FALSE, gtk.TRUE, 0)
		upb = self.app.get_icon_button(gtk.STOCK_GO_UP)
		self.tooltips.set_tip(upb, self.msg['up'])
		upb.connect('clicked', self.edit_up)
		bbox.pack_start(upb, gtk.FALSE, gtk.TRUE, 0)
		dnb = self.app.get_icon_button(gtk.STOCK_GO_DOWN)
		self.tooltips.set_tip(dnb, self.msg['down'])
		dnb.connect('clicked', self.edit_down)
		bbox.pack_start(dnb, gtk.FALSE, gtk.TRUE, 0)
		self.main_vbox.pack_start(bf, gtk.FALSE, gtk.TRUE, 0)
		# Toolbar end

		self.entries = gtk.CList(2, (unicode(_('Name'), 'utf-8'), unicode(_('Script'), 'utf-8')))
		self.entries.set_shadow_type(gtk.SHADOW_IN)
		self.entries.show()
		self.entries.set_selection_mode(gtk.SELECTION_BROWSE)
		self.entries.set_reorderable(gtk.TRUE)
		self.entries.set_column_width(0, 100)
		self.entries.set_column_auto_resize(1, gtk.TRUE)

		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		sw.add(self.entries)
		sw.set_size_request(200, 100)
		sw.show()

		hpane = gtk.HPaned()
		hpane.show()
		hpane.pack1(nf)
		hpane.pack2(sf)

		pane = gtk.VPaned()
		pane.show()
		pane.pack1(sw)
		pane.pack2(hpane)
		self.main_vbox.pack_start(pane, gtk.TRUE, gtk.TRUE, 0)

		buttons = gtk.HBox()
		#buttons.set_size_request(300, 30)
		self.ok  = gtk.Button(unicode(_("OK"), "utf8"), gtk.STOCK_OK)
		self.apl = gtk.Button(unicode(_("Apply"), "utf8"), gtk.STOCK_APPLY)
		self.can = gtk.Button(unicode(_("Cancel"), "utf8"), gtk.STOCK_CANCEL)
		buttons.pack_end(self.can, gtk.FALSE, gtk.TRUE, 0)
		buttons.pack_end(self.apl, gtk.FALSE, gtk.TRUE, 0)
		buttons.pack_end(self.ok, gtk.FALSE, gtk.TRUE, 0)
		buttons.show()
		self.ok.show()
		self.apl.show()
		self.can.show()
		self.main_vbox.pack_end(buttons, gtk.FALSE, gtk.TRUE, 0)

		self.entries.connect("select-row", self.select)
		self.entries.connect("button-press-event", self.popup)
		self.entries.connect("key-press-event", self.keypress)
		self.ok.connect('clicked', self.okey)
		self.apl.connect('clicked', self.apply)
		self.can.connect('clicked', self.cancel)

		self.skey = [gtk.SORT_DESCENDING, gtk.SORT_DESCENDING]
		for i in range(len(self.skey)):
			self.entries.get_column_widget(i).get_parent().connect("clicked", self.sort, i)

		self.initialize()
		self.entries.grab_focus()

	def keypress(self, widget, event):
		if event.keyval == 65535: # delete key
			self.delete(widget, event)

	def popup(self, widget, event):
		sel = self.entries.get_selection_info(int(event.x), int(event.y))
		if sel:
			self.set_focus_row(sel[0], self.entries.rows)
			if event.button == 3:
				self.popup_menu.popup(None, None, None, event.button, 0)
			if event.type == 5:
				self.edit_copy(widget, event)

	def sort(self, widget, colnum):
		if type(widget) is gtk.Button:
			self.skey[colnum] = not self.skey[colnum]

		self.entries.freeze()
		self.entries.set_sort_type(self.skey[colnum])
		self.entries.set_sort_column(colnum)
		self.entries.sort()
		self.entries.thaw()
		self.entries.grab_focus()

		if self.entries.selection:
			self.entries.moveto(self.entries.selection[0], colnum, 0.5, 0)
			self.set_focus_row(self.entries.selection[0], self.entries.rows)

	def edit_up(self, widget=None, event=None):
		if not self.entries.selection:
			return
		row = self.entries.selection[0]
		if row != 0:
			self.entries.swap_rows(row, row-1)
			self.set_focus_row(row-1, self.entries.rows)
			self.apply(widget, event)

	def edit_down(self, widget=None, event=None):
		if not self.entries.selection:
			return
		row = self.entries.selection[0]
		if row+1 < self.entries.rows:
			self.entries.swap_rows(row+1, row)
			self.set_focus_row(row+1, self.entries.rows)
			self.apply(widget, event)

	def edit_duplicate(self, widget=None, event=None):
		if not self.entries.selection:
			return
		row = self.entries.selection[0]
		name = self.entries.get_text(row, 0)
		script = self.entries.get_text(row, 1)
		self.entries.insert(row, (name, script))

	def edit_copy(self, widget=None, event=None):
		if not self.entries.selection:
			return
		script = self.entries.get_text(self.entries.selection[0], 1)
		if not script:
			return
		self.app.insert(script)

	def select(self, clist, row, col, event):
		name   = unicode(self.entries.get_text(row, 0), "utf-8")
		script = unicode(self.entries.get_text(row, 1), "utf-8")
		self.name_text.set_text(name)
		self.script_text.set_text(script)

	def initialize(self):
		self.entries.freeze()
		self.entries.clear()
		for p in self.manager.list():
			p = string.split(p)
			self.entries.append(p)
		self.entries.thaw()
		self.name_text.set_text('')
		self.script_text.set_text('')
		self.entries.select_row(0, 0)
		self.set_focus_row(0, self.entries.rows)
		self.entries.grab_focus()

	def delete(self, widget=None, event=None):
		row = 0
		if self.entries.selection:
			row = self.entries.selection[0]
		self.entries.freeze()
		self.entries.remove(row)
		self.entries.thaw()
		self.set_focus_row(row-1, self.entries.rows)

	def insert_new(self, widget=None, event=None):
		row = 0
		name = ''
		script = ''
		if self.entries.selection:
			row = self.entries.selection[0] + 1
			name   = self.name_text.get_text()
			script = self.script_text.get_text()
		self.entries.freeze()
		self.entries.insert(row, (name, script))
		self.entries.thaw()
		self.set_focus_row(row, self.entries.rows)

	def apply(self, widget=None, event=None):
		if self.entries.selection:
			name   = self.name_text.get_text()
			script = self.script_text.get_text()
			row = self.entries.selection[0]
			self.entries.freeze()
			self.entries.set_text(row, 0, name)
			self.entries.set_text(row, 1, script)
			self.entries.thaw()

		p = []
		for r in range(self.entries.rows):
			n = unicode(self.entries.get_text(r, 0), "utf-8")
			s = unicode(self.entries.get_text(r, 1), "utf-8")
			p.append("%s\t%s" % (n, s))
		self.manager.overwrite(p)
		self.manager.save_database()

	def okey(self, widget=None, event=None):
		self.apply(widget, event)
		self.initialize()
		self.close()

	def cancel(self, widget=None, event=None):
		self.initialize()
		self.close()

	def set_focus_row(self, row, row_count):
		if row_count == 0:
			return
		if row_count == 1:
			r = 0
		else:
			r = min((row+0.00001)/(row_count-1),1.0)
		self.entries.emit("scroll-vertical", gtk.SCROLL_JUMP, r)

	def open(self, widget, data):
		self.window.show()

	def close(self, widget=None, e=None, data=None):
		self.window.hide()
		return gtk.TRUE
