# -*- coding: ascii -*-
#
#  prefs.py - Preference manipulator for GBottler
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: prefs.py,v 1.9 2004/04/29 08:12:10 atzm Exp $
#

import os, sys, re
from gbottler import open_bottlecase

import gtk

class PreferenceDialog:
	def __init__(self, parent, prefmanager, luid_info=None):
		self.parent = parent
		self.prefmanager = prefmanager
		if luid_info:
			self.luid_info = luid_info

		self.window = gtk.Dialog(("GBottler: " + unicode(_("Preferences"))),
								 self.parent,
								 (gtk.DIALOG_DESTROY_WITH_PARENT|gtk.DIALOG_MODAL|gtk.DIALOG_NO_SEPARATOR),
								 (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
								  gtk.STOCK_APPLY, gtk.RESPONSE_APPLY,
								  gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
		self.window.set_border_width(5)
		self.window.connect("destroy", self.close)
		self.window.set_title("GBottler: " + unicode(_("Preferences")))

		self.prefs = {}
		self.prefs['forward'] = gtk.CheckButton(unicode(_("Forward")))
		self.prefs['aa'] = gtk.RadioButton(None, unicode(_("Accept All")))
		self.prefs['foig'] = gtk.RadioButton(self.prefs['aa'], unicode(_("Forward only listed Ghost")))
		self.prefs['ceas'] = gtk.CheckButton(unicode(_("Clear editor after send")))
		self.prefs['auto update'] = gtk.CheckButton(unicode(_("Auto update")))
		for name in ['forward', 'aa', 'foig', 'ceas', 'auto update']:
			self.prefs[name].set_border_width(5)
			self.prefs[name].show()

		self.sendbox = gtk.VBox()
		self.sendbox.show()
		self.sendbox.pack_start(self.prefs['forward'], gtk.FALSE, gtk.FALSE, 0)
		self.prefs['forward'].connect('toggled', self.forward_toggled)
		self.sendbox.pack_start(self.prefs['aa'], gtk.FALSE, gtk.FALSE, 0)
		self.sendbox.pack_start(self.prefs['foig'], gtk.FALSE, gtk.FALSE, 0)

		self.sendframe = gtk.Frame(unicode(_("SSTP server")))
		self.sendframe.show()
		self.sendframe.set_border_width(5)
		self.sendframe.add(self.sendbox)

		self.editorbox = gtk.VBox()
		self.editorbox.show()
		self.editorbox.pack_start(self.prefs['ceas'], gtk.FALSE, gtk.FALSE, 0)
		self.editorframe = gtk.Frame(unicode(_("Main Editor")))
		self.editorframe.show()
		self.editorframe.set_border_width(5)
		self.editorframe.add(self.editorbox)

		self.logbox = gtk.VBox()
		self.logbox.show()
		self.logbox.pack_start(self.prefs['auto update'], gtk.FALSE, gtk.FALSE, 0)
		self.logframe = gtk.Frame(unicode(_("Logs")))
		self.logframe.show()
		self.logframe.set_border_width(5)
		self.logframe.add(self.logbox)

		if luid_info:
			luid_data = self.luid_info.get_all()
			if not luid_data['info'] and not luid_data['table']:
				luid_data = self.luid_info.get_all(True)

			if luid_data['info'] and luid_data['table']:
				self.luidinfolist = gtk.CList(len(luid_data['info'][0]), luid_data['info'][0])
				self.luidinfolist.show()
				self.luidinfolist.set_shadow_type(gtk.SHADOW_IN)
				self.luidinfolist.set_selection_mode(gtk.SELECTION_BROWSE)
				self.luidinfolist.set_reorderable(gtk.TRUE)
				self.luidinfolist.set_column_width(0, 100)
				self.luidinfolist.set_column_auto_resize(1, gtk.TRUE)

				self.luidinfolist.freeze()
				self.luidinfolist.clear()
				for line in luid_data['info'][1:]:
					self.luidinfolist.append(tuple(line))
				self.luidinfolist.thaw()

				self.luidinfosw = gtk.ScrolledWindow()
				self.luidinfosw.show()
				self.luidinfosw.set_size_request(300, 80)
				self.luidinfosw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
				self.luidinfosw.add(self.luidinfolist)

				self.luidinfof = gtk.Frame(unicode(_('General Information')))
				self.luidinfof.set_border_width(5)
				self.luidinfof.show()
				self.luidinfof.add(self.luidinfosw)

				self.luidlist = gtk.CList(len(luid_data['table'][0]), luid_data['table'][0])
				self.luidlist.show()
				self.luidlist.set_shadow_type(gtk.SHADOW_IN)
				self.luidlist.set_selection_mode(gtk.SELECTION_BROWSE)
				self.luidlist.set_reorderable(gtk.TRUE)
				self.luidlist.set_column_width(0, 100)
				self.luidlist.set_column_auto_resize(1, gtk.TRUE)

				self.luidlist.freeze()
				self.luidlist.clear()
				for line in luid_data['table'][1:]:
					self.luidlist.append(tuple(line))
				self.luidlist.thaw()

				self.luidsw = gtk.ScrolledWindow()
				self.luidsw.show()
				self.luidsw.set_size_request(300, 100)
				self.luidsw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
				self.luidsw.add(self.luidlist)

				self.luidf = gtk.Frame(unicode(_('Details')))
				self.luidf.set_border_width(5)
				self.luidf.show()
				self.luidf.add(self.luidsw)

				self.luidbutton = gtk.Button(unicode(_('Update')))
				self.luidbutton.show()
				self.luidbutton.connect('clicked', self.refresh_luid_info)

				self.luidbox = gtk.VBox()
				self.luidbox.show()
				self.luidbox.pack_start(self.luidinfof, gtk.FALSE, gtk.FALSE, 0)
				self.luidbox.pack_start(self.luidf, gtk.TRUE, gtk.TRUE, 0)
				self.luidbox.pack_end(self.luidbutton, gtk.FALSE, gtk.FALSE, 3)

		self.main_notebook = gtk.Notebook()
		self.main_notebook.set_tab_pos(gtk.POS_TOP)
		self.main_notebook.show()
		self.main_notebook.append_page(self.sendframe, gtk.Label(unicode(_("SSTP"))))
		self.main_notebook.append_page(self.editorframe, gtk.Label(unicode(_("Editor"))))
		self.main_notebook.append_page(self.logframe, gtk.Label(unicode(_("Logs"))))
		if luid_info and luid_data['info'] and luid_data['table']:
			self.main_notebook.append_page(self.luidbox, gtk.Label(unicode(_("LUID"))))

		self.window.vbox.pack_start(self.main_notebook, gtk.TRUE, gtk.TRUE, 0)
		self.prefmanager.load_database()
		self.set_prefs()

		self.window.show()
		while True:
			response = self.window.run()

			if response == gtk.RESPONSE_APPLY:
				self.prefmanager.save_database(self)
			elif response == gtk.RESPONSE_ACCEPT:
				self.prefmanager.save_database(self)
				self.close()
				break
			elif response == gtk.RESPONSE_REJECT:
				self.set_prefs()
				self.close()
				break

	def refresh_luid_info(self, widget=None, data=None):
		luid_data = self.luid_info.get_all(True)
		self.luidinfolist.freeze()
		self.luidinfolist.clear()
		for line in luid_data['info'][1:]:
			self.luidinfolist.append(tuple(line))
		self.luidinfolist.thaw()

		self.luidlist.freeze()
		self.luidlist.clear()
		for line in luid_data['table'][1:]:
			self.luidlist.append(tuple(line))
		self.luidlist.thaw()

	def close(self, widget=None, data=None):
		self.window.destroy()

	def get_prefs(self):
		prefs = {}
		for name in ['forward', 'aa', 'foig', 'ceas', 'auto update']:
			prefs[name] = self.prefs[name].get_active()
		return prefs

	def set_prefs(self):
		for key in self.prefmanager.rc.keys():
			self.prefs[key].set_active(self.prefmanager.rc[key])

		if not self.prefs['forward'].get_active():
			self.prefs['aa'].set_sensitive(gtk.FALSE)
			self.prefs['foig'].set_sensitive(gtk.FALSE)

	def forward_toggled(self, widget=None, data=None):
		state = self.prefs['forward'].get_active()
		self.prefs['aa'].set_sensitive(state)
		self.prefs['foig'].set_sensitive(state)

class PreferenceManager:
	DBFILE = "bottlerc"

	def __init__(self):
		self.dbpath = os.path.join(open_bottlecase(), self.DBFILE)
		self.rc = {}
		self.initialize()
		self.load_database()

	def initialize(self):
		for k, v in [['forward', True],
					 ['aa', True],
					 ['foig', False],
					 ['ceas', True],
					 ['auto update', True]]:
			self.rc[k] = v

	def load_database(self):
		pat = re.compile('\s*:\s*')
		try:
			file = open(self.dbpath)
		except IOError:
			return
		while 1:
			line = unicode(file.readline(), "utf-8")
			if not line:
				break
			elif line[0] == "#":
				continue
			elif line[-2:] == "\r\n":
				line = line[:-2]
			elif line[-1] in ["\r", "\n"]:
				line = line[:-1]

			[name, value] = pat.split(line)
			if value == 'True':
				value = True
			elif value == 'False':
				value = False
			self.rc[name] = value
		file.close()

	def save_database(self, prefdialog):
		try:
			file = open(self.dbpath, "w")
		except IOError:
			sys.stderr.write("Error: cannot write %s\n" % self.dbpath)
			return
		self.rc = prefdialog.get_prefs()
		file.write("# This is an auto-generated file.\n")
		file.write("# Don't edit this file while running GBottler!\n")
		for key in self.rc.keys():
			file.write("%s: %s\n" % (key.encode("utf-8"), self.rc[key]))
		file.close()

	def get(self, name):
		if not self.rc.has_key(name):
			raise ValueError("rc has no key %s\n" % name)
		return self.rc[name]
