#!/usr/bin/env python
# -*- coding: ascii -*-
#
#  queryluid.py - Query LUID information
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: queryluid.py,v 1.2 2004/04/25 14:55:01 atzm Exp $
#

import sys, os, string
import httplib
import htmllib, formatter

class LUIDParser(htmllib.HTMLParser):
	def __init__(self):
		htmllib.HTMLParser.__init__(self, formatter.NullFormatter(), 0)

		self.th = False
		self.td = False
		self.tdcount = 0
		self.tddataflag = False

		self.dt = False
		self.dd = False
		self.dtbuf = ''
		self.headres = []
		self.result = []

	def start_dt(self, attrs): self.dt = True
	def end_dt(self):          self.dt = False
	def start_dd(self, attrs): self.dd = True
	def end_dd(self):          self.dd = False
	def start_th(self, attrs): self.th = True
	def end_th(self):          self.th = False
	def start_td(self, attrs): self.td = True
	def end_td(self):
		if not self.tddataflag:
			try:
				self.result[self.tdcount].append('')
			except IndexError:
				self.tdcount = 0
				self.result[self.tdcount].append('')

		self.td = False
		self.tdcount += 1
		self.tddataflag = False

	def handle_data(self, text):
		if self.th:
			self.result.append([text])
		elif self.td:
			self.tddataflag = True
			try:
				self.result[self.tdcount].append(text)
			except IndexError:
				self.tdcount = 0
				self.result[self.tdcount].append(text)

		elif self.dt:
			self.dtbuf += text
		elif self.dd and self.dtbuf:
			self.headres.append([self.dtbuf, text])
			self.dtbuf = ''

	def end_html(self):
		self.format()

	def format(self):
		length = len(self.result[0])
		formatted = [[] for i in range(length)]
		for line in self.result:
			if line[1:]:
				for i in range(length):
					formatted[i].append(line[i])
		self.result = formatted

		length = len(self.headres[0])
		formatted = [[] for i in range(length)]
		for line in self.headres:
			if line[1:]:
				for i in range(length):
					formatted[i].append(line[i])
		self.headres = formatted

	def initialize(self):
		self.th = False
		self.td = False
		self.tdcount = 0
		self.tddataflag = False

		self.dt = False
		self.dd = False
		self.dtbuf = ''
		self.headres = []
		self.result = []

class LUIDInfo:
	HOST = 'bottle.mikage.to'
	AGENT = os.path.basename(sys.argv[0])

	def __init__(self, luid):
		self.luid = luid
		self.parser = LUIDParser()

	def get_html(self):
		try:
			con = httplib.HTTPConnection(self.HOST)
			con.request("GET", '/luidstat.cgi?luid=%s' % self.luid)
			res = con.getresponse()
			html = res.read()
			con.close()
		except:
			return
		return unicode(html, 'sjis', 'ignore')

	def parse(self):
		html = self.get_html()
		if html:
			self.parser.initialize()
			self.parser.feed(html)

	def get_all(self, refresh=False):
		if refresh:
			self.parse()
		return {
			'info':  self.parser.headres,
			'table': self.parser.result,
			}

	def get_info(self, refresh=False):
		if refresh:
			self.parse()
		return self.parser.headres

	def get_table(self, refresh=False):
		if refresh:
			self.parse()
		return self.parser.result
