# -*- coding: ascii -*-
#
#  textmanager.py - TextBuffer manipulator part for GBottler
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: textmanager.py,v 1.7 2004/04/25 08:20:22 atzm Exp $
#

import os
import sys
import gtk

from bottleparser import *
from types import *

class TextManager:
	STYLE_TALK = 1
	STYLE_SCRIPT = 2
	STYLE_SCRIPT_WITH_LINEFEED = 3

	def __init__(self, app, textview, parser=BottleParser('loose')):
		self.app = app
		self.textview = textview
		self.parser = parser
		self.view_style = self.STYLE_TALK
		self.textbuffer = self.textview.get_buffer()
		self.textbuffer.create_tag('script').set_property('foreground', 'dark green')
		self.textbuffer.create_tag('sakura').set_property('foreground', 'black')
		self.textbuffer.create_tag('kero').set_property('foreground', 'brown')
		self.textbuffer.create_tag('sync').set_property('foreground', 'blue')
		self.textbuffer.create_tag('error').set_property('foreground', 'red')

	RE_NEWLINE = re.compile('\n+$')
	def coloring(self, widget=None, event=None):
		current = ['sakura'] # tag stack for closable (currently only synchronized) sessions
		[s, e] = self.textbuffer.get_bounds()

		string = unicode(self.textbuffer.get_text(s, e), 'utf-8')
		string = self.RE_NEWLINE.sub('', string)

		pos = self.textbuffer.get_iter_at_mark(self.textbuffer.get_insert()).get_offset() # cursor position
		end = self.textbuffer.get_end_iter().get_offset()

		chunks = self.parser.parse(string)
		#print 'start:', chunks

		self.textbuffer.set_text("")
		for chunk in chunks:
			if chunk[0] == TEXT_STRING:
				for n in range(len(chunk[1])):
					if chunk[1][n][0] in [SCRIPT_TEXT, TEXT_META]:
						c = chunk[1][n][1]#.replace("\\", r"\\")
						#c = c.replace("%", r"\%")
						#c = c.replace(r"\]", "]")
						self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
															 c, current[-1])
					elif chunk[1][n][0] == PARSE_ERROR:
						c = chunk[1][n][1]#.replace("\\", r"\\")
						self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
															 c, 'error')

			elif chunk[0] in [TEXT_META, PARSE_ERROR]:
				tag = 'script'
				if chunk[0] == PARSE_ERROR:
					tag = 'error'

				name, args = chunk[1], chunk[2:]

				if name in [r'\0', r'\h']:
					if not current[-1] == 'sync':
						current = ['sakura']
				elif name in [r'\1', r'\u']:
					if not current[-1] == 'sync':
						current = ['kero']
				elif name == r'\_s':
					if current[-1] == 'sync' and len(current) > 1:
						current.pop()
						# when synchronized session closed
					else:
						current.append('sync')

				self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
													 name, tag)

				if name == r'\URL' and chunk[0] == PARSE_ERROR and len(args) >= 2:
					format = '[%s]'
				elif name == r'\w' or (name in [r'\s', r'\URL'] and chunk[0] == PARSE_ERROR):
					format = '%s'
				else:
					format = '[%s]'

				for n in range(len(args)):
					if type(args[0]) is TupleType:
						try:
							if args[n][0][0] == PARSE_ERROR:# and name == r'\n':
								self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																	 "%s" % args[n][0][1], 'error')
							else:
								self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																	 format % args[n][0][1], tag)
						except IndexError:
							pass
					else:
						self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
															 format % args[n], tag)

		self.textbuffer.insert(self.textbuffer.get_end_iter(), '\n')
		self.textview.emit('move-cursor', gtk.MOVEMENT_LOGICAL_POSITIONS, end*-2, gtk.FALSE)
		self.textview.emit('move-cursor', gtk.MOVEMENT_LOGICAL_POSITIONS, pos, gtk.FALSE)

	def insert_with_color_from(self, string):
		current = ['sakura'] # tag stack for closable (currently only synchronized) sessions
		for chunk in self.parser.parse(string):
			if chunk[0] == TEXT_STRING:
				if chunk[1][0][0] == SCRIPT_TEXT:
					self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
															 chunk[1][0][1], current[-1])

			elif chunk[0] == TEXT_META:
				name, args = chunk[1], chunk[2:]

				if name in [r'\0', r'\h']:
					if not current[-1] == 'sync':
						current = ['sakura']
				elif name in [r'\1', r'\u']:
					if not current[-1] == 'sync':
						current = ['kero']
				elif name == r'\_s':
					if current[-1] == 'sync' and len(current) > 1:
						current.pop()
						# when synchronized session closed
					else:
						current.append('sync')

				if self.view_style == self.STYLE_TALK:
					pre_end = self.textbuffer.get_end_iter()
					first = pre_end.backward_char()
					if name in [r'\0', r'\h', r'\1', r'\u', r'\_s'] and \
						   pre_end.get_char() != '\n' and first:
						if current[-1] != 'sync' or name == r'\_s':
							self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																	 '\n', 'script')
					elif name == r'\URL':
						for n in range(len(args)):
							self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																	 "[%s]" % args[n][0][1], 'script')

				else:
					self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
															 name, 'script')
					if name == r'\w':
						format = '%s'
					else:
						format = '[%s]'

					for n in range(len(args)):
						if type(args[0]) is TupleType:
							self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																	 format % args[n][0][1], 'script')
						else:
							self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																	 format % args[n], 'script')

					if self.view_style == self.STYLE_SCRIPT_WITH_LINEFEED and name == r'\n':
						self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
																 '\n', 'script')

		self.textbuffer.insert_with_tags_by_name(self.textbuffer.get_end_iter(),
												 '\n', 'script')
		self.textview.emit("move-cursor", gtk.MOVEMENT_LOGICAL_POSITIONS, -3, gtk.FALSE)

	def set_style_talk(self):
		self.view_style = self.STYLE_TALK

	def set_style_script(self):
		self.view_style = self.STYLE_SCRIPT

	def set_style_script_with_linefeed(self):
		self.view_style = self.STYLE_SCRIPT_WITH_LINEFEED

	def set_style(self, style):
		if type(style) is not IntType:
			raise ValueError("Invalid argument: " + str(style))
		self.view_style = style
